/*
 *  Copyright (c) 2017 - present, Architecode Corporation. All Rights Reserved.
 *
 *  This source code is licensed under the MIT License.
 *  The LICENSE file can be found in the root directory.
 *
 *  @cix-lic
*/


"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../errors");
class LoggerRegistry {
    constructor(options = {}) {
        Object.defineProperties(this, {
            "UseInstanceCache": {
                configurable: false,
                enumerable: false,
                writable: false,
                value: options.useInstanceCache != undefined ? options.useInstanceCache : true
            },
            "DefaultLogger": {
                configurable: false,
                enumerable: false,
                writable: false,
                value: {}
            },
            "ModulesMap": {
                configurable: false,
                enumerable: false,
                writable: false,
                value: new Map()
            },
            "SetupsMap": {
                configurable: false,
                enumerable: false,
                writable: false,
                value: new Map()
            },
        });
    }
    getDefaultLogger() {
        return this.DefaultLogger;
    }
    setDefaultLogger(logger, properties) {
        this.DefaultLogger.logger = logger;
        this.DefaultLogger.properties = properties;
    }
    getLoggerModule(logger) {
        return this.ModulesMap.get(logger);
    }
    setLoggerModule(logger, LoggerModule) {
        this.ModulesMap.set(logger, LoggerModule);
    }
    hasLoggerModule(logger) {
        return this.ModulesMap.has(logger);
    }
    getLoggerSetup(name, type) {
        if (this.SetupsMap.has(type) && this.SetupsMap.get(type).has(name)) {
            const { logger, properties } = this.SetupsMap.get(type).get(name);
            return { name, type, logger, properties };
        }
        else {
            return {};
        }
    }
    setLoggerSetup(name, type, logger, properties) {
        if (!this.SetupsMap.has(type)) {
            this.SetupsMap.set(type, new Map());
        }
        this.SetupsMap.get(type).set(name, { logger, properties });
    }
    hasLoggerSetup(name, type) {
        return this.SetupsMap.has(type) && this.SetupsMap.get(type).has(name);
    }
    resolveDefaultLogger(name, type, properties) {
        const Logger = this.getLoggerModule(this.DefaultLogger.logger);
        if (Logger) {
            return new Logger(name, type, this.DefaultLogger.logger, properties || this.DefaultLogger.properties);
        }
        else {
            throw new errors_1.UndefinedDefaultLoggerError();
        }
    }
    resolveLogger(name, type) {
        if (this.hasLoggerSetup(name, type)) {
            const typeMap = this.SetupsMap.get(type);
            const setup = typeMap.get(name);
            if (this.UseInstanceCache && setup.instance) {
                return setup.instance;
            }
            else {
                const { logger, properties } = setup;
                let instance;
                if (logger && this.hasLoggerModule(logger)) {
                    const Logger = this.getLoggerModule(logger);
                    instance = new Logger(name, type, logger, properties);
                }
                else {
                    instance = this.resolveDefaultLogger(name, type, properties);
                }
                if (this.UseInstanceCache) {
                    typeMap.set(name, { logger, properties, instance });
                }
                return instance;
            }
        }
        else {
            const instance = this.resolveDefaultLogger(name, type);
            if (this.UseInstanceCache) {
                if (!this.SetupsMap.has(type)) {
                    this.SetupsMap.set(type, new Map());
                }
                const typeMap = this.SetupsMap.get(type);
                typeMap.set(name, { instance });
            }
            return instance;
        }
    }
}
exports.LoggerRegistry = LoggerRegistry;
